#!/usr/bin/perl
#
# fraglets0.17/bin/gencode.pl
#
# automatic generation of short code fragments guaranteed to be correct,
# using a deterministic method
#
# Copyright (C) 2005 <lidia.yamamoto@unibas.ch>
#
# This file is distributed under the terms of the GNU general Public
# License (GPL), see the file COPYING
#
# Usage:
#
# input format (stdin):
#   <tag> <symbol>
#   <frag1>
#   <frag2>
#   ...
#
# <tag> = name of tag to be matched, ie. first instruction will be
#   f n[matchp tag ...]
#
# <symbol> = input symbol to be provided to <tag> for reaction,
#   in the form n[tag : symbol]
#   (currently only one input symbol is supported)
#
# <frag1>... = desired result fraglets (order not important)
#   for the moment only one result fraglet is supported
#   multiplicity currently not supported
#
# output (stdout) is the generated code for the node specified in the
# result fraglet.
#
# History:
# 20050206 creation by lidia.yamamoto@unibas.ch
#


sub parse {
    while (<>) {
	if ( /^f[ \t]+([a-zA-Z0-9_]+)\[(.*)\]/ ) {
	    $node=$1;
	    $fcore=$2;
	    @symlist = split(/:/, $fcore);

	    for ($i=0; $i <= $#symlist; $i++) {
		$symlist[$i] =~ s/^[ \t]+//; $symlist[$i] =~ s/[ \t]+$//;
		#print "sym=$symlist[$i]\n";
		if ( $symlist[$i] =~ /[ \t]/ ) {
		    print STDERR "gencode.pl: syntax error: missing \"\:\" in <$symlist[$i]>\n";
                    exit -1;
		}
            }
	} elsif ( /([a-zA-Z0-9_]+)[ \t]+([a-zA-Z0-9_]+)/ ) {
	    $tag=$1;
	    $isym=$2;
	} else {
	    print STDERR "gencode.pl: wrong input format\n";
	    exit -1;
	}
    }
    if ($node eq "" || $fcore eq "" || $tag eq "" || $isym eq "") {
	print STDERR "gencode.pl: missing information\n";
	exit -1;
    }
    #print "node=$node, tag=$tag, isym=$isym, fcore=$fcore\n";
}


$idgen=0;
sub newsymbol {
    $idgen++;
    return "${tag}_$idgen";
}


$ip=0;
$fc=0;
$prevtag="";

sub gencode {
    $frag = "";
    for ($i=$ip; $i <= $#symlist && $symlist[$i] ne $isym; $i++) {
	#print "isym=<$isym>  symlist[$i]=<$symlist[$i]>\n";
	$frag = "$frag  : $symlist[$i]";
    }
    if ($frag ne "") {
	$news = newsymbol();
	$prevtag = $news;
	$flist[$fc++] = "matchp : $news $frag";
	$ip = $i;
    } elsif ($symlist[$ip] eq $isym) {
	if ($ip < $#symlist) {
	    $next = $symlist[$ip+1];
	    $news = newsymbol();
	    if ($next eq $isym) {
		$flist[$fc++] = "matchp : $news : dup : $prevtag";
	    } else {
		$flist[$fc++] = "matchp : $news : exch : $prevtag : $next";
		$symlist[$ip+1] = $isym;
	    }
	    $ip++;
	    $prevtag = $news;
	} else {
	    $ip++;
	}
    }
    if ($ip > $#symlist) {
	$flist[$fc-1] =~ s/$prevtag/$tag/;
    }
}


sub print_fraglets {
    for ($i=$fc-1; $i >= 0; $i--) {
	print "f $node\[ $flist[$i] \]\n";
    }
}


&parse();

while ($ip <= $#symlist) {
    &gencode();
}

&print_fraglets();

